/*
-------------------------------------------------------------------Title: CoV Contrast Test Macro-----------------------------------------------------------------
Version: 1.5
Updated: 4 May 2018

Description:
- Computes the Root Mean Sqaure Contast of the area of interest defined by the user: individual ROIs, all ROIs combined, or the
entire image.

Requirements:
- Image to be analysed should be open before running this macro
- If user wants to analyse individual ROIs or all ROIs combined: one ROI around each area of interest
  If whole image is to be analysed: no special requirements (ROIs present in the image will be ignored)
- No other ROIs (e.g. scale bars) in the image
-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
*/

print("\\Clear");
print("-----RMS Contrast Macro Initiated-----\nGathering options ...");

//TO DO: Make it batch friendly 

// Dialog to determine what to analyse
items = newArray("Whole image", "ROIs combined", "ROIs individually");
Dialog.create("Analysis Options");
Dialog.addChoice("What do you want to calculate the Contrast for?", items);
if(nSlices>1){
	Dialog.setInsets(5,0,0);
	Dialog.addMessage("----------------------")
	Dialog.setInsets(5,0,0);
	Dialog.addMessage("Multiple slices have been detected in your image. \nPlease check all the slices that you want to analyse:")
	sliceNames = newArray(nSlices);
	for(i=1;i<=nSlices;i++){
		setSlice(i);
		sliceNames[i-1]=getInfo("slice.label");
		Dialog.addCheckbox(sliceNames[i-1], true);
	}
}
Dialog.show();
choice = Dialog.getChoice(); 
if(nSlices>1){
	sliceChoices = newArray(nSlices);
	for(i=0;i<nSlices;i++){
		sliceChoices[i] = Dialog.getCheckbox();
	}
}

// Check number of ROIs
nROIs = nLoops = roiManager("count");
if(choice=="ROIs combined" || choice=="ROIs individually"){
	if(nROIs==0){
		print("Error: no ROIs detected in the ROI Manager.\n-----Macro Terminated-----");
		selectWindow("Log");
		exit("Error: no ROIs detected. Add ROIs and restart this macro.");
	}	
}
if(nLoops == 0)
	nLoops = 1;

// Determine which slices need to be analysed and create array to store results
if(nSlices==1){
	numSlices = 1;
	rmsSaveArea = newArray(nLoops);
	rmsSaveSD = newArray(nLoops);
	slicesToAnalyse = newArray(1);
	slicesToAnalyse[0] = 1;
	sliceNames = newArray(1);
	sliceNames[0] = getTitle();
}else{
	numSlices = 0;
	for(i=0; i<lengthOf(sliceChoices); i++){
		if(sliceChoices[i] == true)
			numSlices++;
	}
	if(numSlices==0)
		exit("Error: No slices selected.");
	rmsSaveArea = newArray(nLoops*numSlices);
	rmsSaveSD = newArray(nLoops*numSlices);

	// Determine which slices to analyse
	slicesToAnalyse = newArray(numSlices);
	temp = 0;
	for(i=1; i<=lengthOf(sliceChoices); i++){
		if(sliceChoices[i-1] == true){
			slicesToAnalyse[temp] = i;
			temp++;
		}
	}
}

run("8-bit");

if(choice=="Whole image"){
	print("Calculating RMS Contrast for whole image ...");

	waitForUser("IMPORTANT: Ensure that no ROIs or selections are currently active!");
	for(i=0; i<numSlices; i++){
		setSlice(slicesToAnalyse[i]);
		getStatistics(area, mean, min, max, std);
		rmsSaveArea[i] = area;
		rmsSaveSD[i] = std/(mean*255);
		getRawStatistics(nPixels);
		print(nPixels);
	}

/* Alternative: safer but takes way longer!
	width = getWidth();
	height = getHeight();
	pixelArray = newArray(width*height);
	for(i=0; i<numSlices; i++){
		setSlice(slicesToAnalyse[i]);
		for(j=0; j<height; j++){
			for(k=0; k<width; k++){
				pixelArray[(j*width)+k] = getPixel(k,j);
			}
		}
		Array.getStatistics(pixelArray, min, max, mean, std);
		rmsSaveArea[i] = width*height;
//		rmsSaveSD[i] = std/255;
		rmsSaveSD[i] = std/(mean*255);
	}
*/


	// Calculate area-weighted, normalised (between 0 to 1) RMS Contrast
	totalArea = 0;
	for (i=0; i<numSlices; i++)
		totalArea = rmsSaveArea[i] + totalArea;
	rmsCombi = 0;
	for (i=0; i<numSlices; i++)
		rmsCombi = rmsSaveSD[i]*rmsSaveArea[i]/totalArea + rmsCombi;

	// Print RMS Contrast to Results Table
	run("Clear Results");
	for(i=0; i<numSlices; i++){
		setResult("RMS Contrast", i, rmsSaveSD[i]);
		setResult("Label", i, sliceNames[slicesToAnalyse[i]-1]);
	}
	setResult("RMS Contrast", numSlices, rmsCombi);
	setResult("Label", numSlices, "Mean (of analysed slices only)");

	print("...\nRMS Contrast for the entire image printed to Results window.\n-----End of Macro-----");
}

if(choice=="ROIs combined"){
	print("Calculating RMS Contrast for ROIs combined ...");

	// Calculate area-weighted, normalised (between 0 to 1) RMS Contrast and print to Results table
	totalArea = 0;
	rmsCombiROI = 0;
	rmsCombiSlice = 0;
	run("Clear Results");
	for(i=0; i<numSlices; i++){
		totalArea = 0;
		rmsCombiROI = 0;
		for(j=0; j<nLoops; j++){
			roiManager("select", j);
			setSlice(slicesToAnalyse[i]);
			getStatistics(area, mean, min, max, std);
			rmsSaveArea[(nLoops*i)+j] = area;
			rmsSaveSD[(nLoops*i)+j] = std/(mean*255);
			totalArea = totalArea + area;
		}
		for(j=0; j<nLoops; j++)
			rmsCombiROI = rmsSaveSD[(nLoops*i)+j]*rmsSaveArea[(nLoops*i)+j]/totalArea + rmsCombiROI;
		setResult("RMS Contrast", i, rmsCombiROI);
		setResult("Label", i, "ROIs in "+sliceNames[slicesToAnalyse[i]-1]);
		rmsCombiSlice = rmsCombiROI/numSlices + rmsCombiSlice;
	}
	setResult("RMS Contrast", numSlices, rmsCombiSlice);
	setResult("Label", numSlices, "Mean (of slices analysed only)");

	print("...\nRMS Contrast for ROIs combined printed to Results window.\n-----End of Macro-----");
}


if(choice=="ROIs individually"){
	print("Calculating RMS Contrast for ROIs individually ...");

	roiNames = newArray(nROIs);

	// Get the RMS contrast for each ROI
	for(j=0; j<nLoops; j++){
		roiManager("select", j);
		roiNames[j] = getInfo("selection.name");	
		for(i=1; i<=numSlices; i++){
			setSlice(slicesToAnalyse[i-1]);
			getStatistics(area, mean, min, max, std);
			rmsSaveSD[(numSlices*j)+(i-1)] = std/(mean*255);
		}
	}

	rmsCombi = 0;

	// Calculate area-weighted, normalised (between 0 to 1) RMS Contrast and print to Results table
	run("Clear Results");
	for(j=0; j<nLoops; j++){
		rmsCombi = 0;
		for(i=0; i<numSlices; i++){
//			setResult("ROI"+(j+1), i, rmsSaveSD[(numSlices*j)+i]);
			setResult(roiNames[j], i, rmsSaveSD[(numSlices*j)+i]);
			rmsCombi = (rmsSaveSD[(numSlices*j)+i]) + rmsCombi;
		}
//		setResult("ROI"+(j+1), numSlices, rmsCombi/numSlices);
		setResult(roiNames[j], numSlices, rmsCombi/numSlices);
	}

	for(j=0; j<numSlices; j++)
		setResult("Label",j, sliceNames[slicesToAnalyse[j]-1]);
	setResult("Label",numSlices,"Mean (of slices analysed only)");

	print("...\nRMS Contrast for each ROI printed to Results window \n(in the same order as the ROIs in the ROI Manager).\n-----End of Macro-----");
}
updateResults();
selectWindow("Log");
selectWindow("Results");
exit;
//exit("Analysis complete. See Results Table.");

/* For troubleshooting
print("rmsSaveArea for Slice "+slicesToAnalyse[i]);
Array.print(rmsSaveArea);
print("rmsSaveSD for Slice "+slicesToAnalyse[i]);
Array.print(rmsSaveSD);
print("Total Area for Slice "+slicesToAnalyse[i]);
print(totalArea);
print("rmsCombiROI for Slice "+slicesToAnalyse[i]);
print(rmsCombiROI);
print("rmsCombiSlice for Slice "+slicesToAnalyse[i]);
print(rmsCombiSlice);
*/
