/*
-------------------------------------------------------------------Title: Distribution Directionality Calculator Macro---------------------------------------------------------------
Version: 1.3
Updated: 6 May 2018

Description: 
- This macro calculates the directionality in the distribution of the markings in a pattern by plotting the centroids of all the markings, drawing a
Line of Best Fit, and then calculating its R^2 value. A high R^2 value would mean that the markings are distributed in a highly-directional manner. 
- Under the right conditions, the slope of the Best Fit Line may also give an indication of the orientation of this directionality. Take note that the
rotation of the images would have had to be standardised prior to using this macro.

Requirements:
- One ROI around each marking
- No other ROIs (e.g. scale bars) in the image
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
*/

print("\\Clear");
print("-----Distribution Directionality Calculator Macro Initiated-----\nCalculating distribution directionality ...");
selectWindow("Log");

run("Clear Results");
run("Set Measurements...", "centroid redirect=None decimal=3");
nROIs = roiManager("count");

if (nROIs==0){
	print("Error: no ROIs were detected. Make sure there are ROIs in your image and restart this macro.\n-----Macro Terminated-----");
	exit("Error: no ROIs were detected.");
}

for(i=0; i<nROIs; i++){
	roiManager("select", i);

/* For some reason makes everything not work
	// Check if Selected ROI is an Area ROI
	if (selectionType==-1 || selectionType==5 || selectionType==6  || selectionType==7 || selectionType==8 || selectionType==10){
		print("ROI "+i+" is not an area selection, skipping to next ROI");
		continue;
	}
*/

	// Get centroid for Selected ROI
	roiManager("measure");
}

/* For some reason also makes everything not work
// Reset to default measurements
run("Set Measurements...", "area mean min redirect=None decimal=3");
*/

// Create arrays to store X and Y Coordinates
x = newArray(nResults);
y = newArray(nResults);

// Copy X and Y Coordinates from Results table
for(i=0; i<nResults; i++){
	x[i] = getResult("X", i);
	y[i] = getResult("Y", i);
}

// Plot Line of Best Fit and Print Equation and R^2 Value
Fit.doFit("Straight Line", x, y);
Fit.plot;

m = d2s(Fit.p(1), 3);
c = d2s(Fit.p(0), 3);
x1 = x[0];
x2 = x[nResults-1];
y1 = (x1*m)+c;
y2 = (x2*m)+c;
direction = getAngle(x1, y1, x2, y2);
if(direction>=180)
	direction = direction-180;

//to convert the coefficient into degrees: horizontal = 0deg, upwards is +ve, downwards is -ve. 
// Eqn: y = 21*ln(x) + 45, but not perfect
//direction = 21*ln(-1*d2s(Fit.p(1), 3))+45; 
run("Clear Results");

//setResult("Value", 0, -1*d2s(Fit.p(1), 3));
setResult("Value", 0, direction);
setResult("Value", 1, d2s(Fit.rSquared, 3));
eqn = "y = "+d2s(Fit.p(1), 3)+"x + "+d2s(Fit.p(0), 3);
setResult("Value", 2, eqn);

rowNames = newArray("Angle (Degrees)", "Alignment (R^2 Value)", "Equation of Line of Best Fit");
for(i=0; i<3; i++)
	setResult("Label", i, rowNames[i]);
updateResults();

//print("R^2 value: "+d2s(Fit.rSquared, 3));
//print("Slope of Best Fit Line: "+d2s(Fit.p(1), 3));
//print("Equation of Best Fit Line: y = "+d2s(Fit.p(1), 3)+"x + "+d2s(Fit.p(0), 3));

print("...\nLine of best fit plotted and Distribution Directionality results displayed in Results window.");
print("...");
//print("Note: The 'Direction' metric represents the slope of the line as we see it on the image. This\nisopposite to the coefficient of X in the equation because X coordinates increase downwards \nin the image but upwards in the graph.\n-----End of Macro-----");
print("Note: The 'Direction' metric is the angle of the line of best fit from the horizontal axis (rotating anticlockwise).\n-----End of Macro-----");
selectWindow("Results");
selectWindow("Log");
exit;

// Returns the angle in degrees between the specified line and the horizontal axis.
function getAngle(x1, y1, x2, y2) {
	q1=0; q2orq3=2; q4=3; //quadrant
	dx = x2-x1;
	dy = y1-y2;
	if (dx!=0)
	angle = atan(dy/dx);
	else {
		if (dy>=0)
			angle = PI/2;
	else
		angle = -PI/2;
	}
	angle = (180/PI)*angle;
	if (dx>=0 && dy>=0)
		quadrant = q1;
	else if (dx<0)
		quadrant = q2orq3;
	else
		quadrant = q4;
	if (quadrant==q2orq3)
		angle = angle+180.0;
	else if (quadrant==q4)
		angle = angle+360.0;
	return angle;
}
