/*
-------------------------------------------------------------------Title: Shape Directionality Calculator Macro---------------------------------------------------------------
Version: 1.2
Updated: 25 Nov 2016

Description:
- Measure the directionality of the markings in a pattern by calculating the ratio between the major and minor axes.
- Also gives the orientation of the long axes which gives an idea of how aligned the markings are (if they are all aligned, the pattern is more
directional.

Requirements:
- One ROI around each marking to be analysed
- No other ROIs (e.g. scale bars) in the image
------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
*/

print("\\Clear");
print("-----Shape Directionality Calculator Macro Initiated-----\nCalculating shape directionality ...");

run("Clear Results");
nROIs = roiManager("count");
roiNames = newArray(nROIs);

if (nROIs==0){
	print("Error: no ROIs were detected. Make sure there are ROIs in your image and restart this macro.\n-----Macro Terminated-----");
	exit("Error: no ROIs were detected.");
}

// Create arrays to save aspect ratio and direction results
saveAR = newArray(nROIs);
saveAngle = newArray(nROIs);

for(i=0; i<nROIs; i++){
	
	roiManager("select", i);
	roiNames[i] = getInfo("selection.name");

	/* For some reason makes everything not work
	// Check if Selected ROI is an Area ROI
	if (selectionType==-1 || selectionType==5 || selectionType==6  || selectionType==7 || selectionType==8 || selectionType==10){
		print("ROI "+i+" is not an area selection, skipping to next ROI");
		continue;
	}
	*/

	// Get Measurements for Selected ROI
	List.setMeasurements;
	//print(List.getList); // list all measurements
	x = List.getValue("X");
	y = List.getValue("Y");
	a = List.getValue("Major");
	b = List.getValue("Minor");
	angle = List.getValue("Angle");

	// Draw ellipse for user to visualise
	getVoxelSize(w, h, d, unit);
	drawEllipse(x/w, y/w, (a/w)/2, (b/w)/2, angle);

	// Print measurements to Results window and save in results arrays
	ar = a/b;
	setResult("Aspect Ratio", i, ar);
	setResult("Angle", i, angle);
	setResult("Label", i, getInfo("selection.name"));
	saveAR[i] = ar;
	saveAngle[i] = angle;
}

//Print Mean and SD to Results window
Array.getStatistics(saveAR, min, max, mean, sd);
setResult("Aspect Ratio", nROIs, mean);
setResult("Label", nROIs, "Mean");
setResult("Aspect Ratio", nROIs+1, sd);
setResult("Label", nROIs+1, "SD");
Array.getStatistics(saveAngle, min, max, mean, sd);
setResult("Angle", nROIs, mean);
setResult("Label", nROIs, "Mean");
setResult("Angle", nROIs+1, sd);
setResult("Label", nROIs+1, "SD");

updateResults();

print("...\nShape Directionality measures printed to Results window.\n-----End of Macro-----");
selectWindow("Log");
selectWindow("Results");
exit;

function drawEllipse(x, y, a, b, angle) {
	autoUpdate(false);
	setLineWidth(2);
	beta = -angle * (PI/180);
	for (i=0; i<=360; i+=2) {
		alpha = i*(PI/180) ;
		X = x + a*cos(alpha)*cos(beta) - b*sin(alpha)*sin(beta);
		Y = y + a*cos(alpha)*sin(beta) + b*sin(alpha)*cos(beta);
		if (i==0) moveTo(X, Y); else lineTo(X,Y);
		if (i==0) {ax1=X; ay1=Y;}
		if (i==90) {bx1=X; by1=Y;}
		if (i==180) {ax2=X; ay2=Y;}
		if (i==270) {bx2=X; by2=Y;}
	}
	drawLine(ax1, ay1, ax2, ay2);
	drawLine(bx1, by1, bx2, by2);
	updateDisplay;
}


