/*
---------------------------------------------------Title: Marking Matrix Macro---------------------------------------------------
Version: 1.5
Updated: 9 Oct 2018

Description:
- This macro will take an image or ROIs  that is/are already thresholded by the user (black for markings,
white for background), and convert it into a matrix of '1's (for markings) and 0's (background).
- This macro can work on either a whole image, an individual ROI or combine multiple ROIs (it will produce 
a cumulative matrix for all the ROIs).

Requirements:
- You need to have already thresholded your image to black (for markings) and white (for the 
background), e.g. by thresholding (Image>Adjust>Threshold), and drawn a rectangle ROI around
the area to be analysed.
- If analysing more than one ROI, all ROIs MUST BE the exact same size.
-------------------------------------------------------------------------------------------------------------------------------------------
*/
/* TO DO: make it batch, and allow multiple images to be combined into a cumulative matrix*/
/* File already exists dialogue*/
/* ROIs from different images
Whole Image and Indiv/Cumulative/Frequency/HM: manually or all images in a folder
ROIs in different images: process all Rois in an image, use getBoolean to see if this is the final image, wait for user to open the next image and load the ROIs, repeat
*/

print("\\Clear");
print("-----Marking Matrix Macro Initiated-----\nGetting analysis options ...");
labels = newArray("Individual Matrices", "Cumulative Matrix", "Frequency Matrix", "Heat Map");

info = "<html>"
   +"Options info<br> <br>"
   +"Individual Matrices:<br>"
   +"-If 'Whole Image' is selected, one pixel matrix for <b>the entire</b> <br>"
   +"     <b>image</b> will be saved as an excel file.<br> 
   +"-If 'ROIs' is selected, the pixel matrices for <b>every ROI</b> in the<br>"
   +"     ROI Manager will be saved as separate excel files.<br> <br>"
   +"Cumulative Matrix:<br>"
   +"-A pixel matrix that is the <b>sum</b> of all the individual matrices will<br>" 
   +"     be saved as an excel file.<br>"
   +"-Not applicable if you are analysing only one ROI or image.<br> <br>"
   +"Frequency Matrix:<br>"
   +"-A pixel matrix that is the <b>average</b> of all the individual matrices<br>" 
   +"     will be saved as an excel file.<br>"
   +"-Not applicable if you are analysing only one ROI or image.<br> <br>"
   +"Heat Map:<br>"
   +"-A 11-level heat map (blue '0' to white '5'  to red '10') will be displayed and<br>"
   +"     saved as a jpeg file.<br>"
   +"-Not applicable if you are analysing only one ROI or image.<br> <br>"
   +"Note: all files will be saved in the folder that your image is in,<br>" 
   +"     within a subfolder named 'Pixel_Matrix'."

// Get user to select analysis options
items = newArray("Whole image", "ROI(s) in the image");
items2 = newArray("Single Image", "Multiple Images");
Dialog.create("Analysis Options");
Dialog.setInsets(5,0,0);
//Dialog.addChoice("Are you analysing a whole image or ROI(s) in the image?", items);
Dialog.addRadioButtonGroup("1) Are you analysing a whole image or ROI(s) in the image?", items, 1, 2, "Whole image");
Dialog.setInsets(10,0,0);
//Dialog.addMessage("Do you need to analyse the image/ROI(s) from a single image or multiple images?");
//Dialog.addChoice("", items2);
Dialog.addRadioButtonGroup("2) Do you need to analyse the image/ROI(s) from a single image or multiple images?", items2, 1, 2, "Single Image");
Dialog.setInsets(10,0,0);
Dialog.addMessage("3) Output options (see 'Help' for details):");
Dialog.setInsets(0,5,0);
Dialog.addCheckboxGroup(1,4,labels,newArray(1,0,0,0));
Dialog.setInsets(20,0,0);
Dialog.addMessage("IMPORTANT:") 
Dialog.setInsets(0,0,0);
Dialog.addMessage("- If analysing only one image/ROI, cumulative/frequency matrix and heat maps will not work.");
Dialog.setInsets(0,0,0);
Dialog.addMessage("- If analysing multiple images/ROIs, all of them need to have the same dimensions.");
Dialog.setInsets(20,150,0);
Dialog.addMessage("Note: this macro can take awhile if you have many large ROIs");
//Dialog.setInsets(10,5,0);
//Dialog.addCheckbox("My ROIs are in different images:",0);
//Dialog.setInsets(0,5,0);
//Dialog.addMessage("Select this if you want to produce a cumulative/frequency matrix or heat map from\nmultiple images or ROIs in multiple images..");
Dialog.addHelp(info);
Dialog.show();
//choice = Dialog.getChoice(); 
choice=Dialog.getRadioButton();
//choice2 = Dialog.getChoice(); 
choice2=Dialog.getRadioButton();
outputIn = Dialog.getCheckbox(); //individual
outputCu = Dialog.getCheckbox(); //cumulative
outputFr = Dialog.getCheckbox(); //frequency
outputHM = Dialog.getCheckbox(); //heat map
//multiple = Dialog.getCheckbox();

// Check that analysis options are valid
nROIs = roiManager("count");
// not valid if: Whole Image and Single Image are selected, and Cu, Fr or HM is/are selected
// ROIs and Single Image are selected and there is only one ROI, and Cu, Fr or HM is/are selected
check2 = 1;
invalids="";

if(outputCu==1||outputFr==1||outputHM==1){
	if(choice2=="Single Image"){
		if(choice=="Whole image"){
			if(outputCu==1)
				invalids=invalids+"<Cumulative Matrix> ";
			if(outputFr==1)
				invalids=invalids+"<Frequency Matrix> ";
			if(outputHM==1)
				invalids=invalids+"<Heat Map>";
			outputCu = 0;
			outputFr = 0;
			outputHM = 0;
			check2 = 0;
		}
		if(choice=="ROI(s) in the image" && nROIs==1){
			if(outputCu==1)
				invalids=invalids+"<Cumulative Matrix> ";
			if(outputFr==1)
				invalids=invalids+"<Frequency Matrix> ";
			if(outputHM==1)
				invalids=invalids+"<Heat Map>";
			outputCu = 0;
			outputFr = 0;
			outputHM = 0;
			check2 = 0;
		}
	}
}

if(check2==0)
	print("Warning: there is only one ROI/image to analyse and cumulative/\nfrequency matrices or heat maps cannot be produced. The invalid\noptions below have been deselected:\n"+invalids);

// Check that at least one analysis option is selected
check1 = outputIn+outputCu+outputFr+outputHM;
if(check1==0){
	print("Error: no valid analysis options were selected. Restart the macro\nand select at least one analysis option.");
	print("-----Macro terminated-----");
	selectWindow("Log");
	exit("Error: see Log for details.");	
}

if(choice=="Whole Image" && choice2=="Single Image"){
	print("Performing analysis ...");
	// Get the dimensions of the image and create results array
	getDimensions(width, height, channels, slices, frames);
	resultsArray = newArray((width+1)*(height+1));
	// Perform analysis
	run("8-bit");
	for(j=0; j<height; j++){
		for(k=0; k<width; k++){
			resultsArray[(j*width)+k] = getPixel(k,j)/255;
		}
	}
	// Update results table
	for(row=0; row<height; row++){
		for(col=0; col<width; col++){
			setResult(col+1,row, resultsArray[(row*width)+col]);
		}
	}
	setOption("ShowRowNumbers", true);
	updateResults();
	// Save to file
	dir = getDirectory("image")+"Pixel_Matrix"+File.separator;
	File.makeDirectory(dir);
	fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."));
	// Note: row and column headers are also saved in the xls file
	saveAs("results", dir+fileName+".xls");
}

if(choice=="Whole Image" && choice2=="Multiple Images"){
	items3 = newArray("All images in a folder","Manually open images");
	Dialog.create("Analysis Options");
	Dialog.addRadioButtonGroup("How do you want to analyse your images?:",items3,2,1,"All images in a folder");
	Dialog.setInsets(10,10,0);
	Dialog.addMessage("IMPORTANT:");
	Dialog.setInsets(0,10,0);
	Dialog.addMessage("- All images must have the same dimensions");
	Dialog.setInsets(0,10,0);
	Dialog.addMessage("- If 'All images in a folder' is selected, images\nin subfolders will not be analysed");
	Dialog.show;
	choice3 = Dialog.getRadioButton();
	print("Performing analysis ...");
	masterWidth=0;
	masterHeight=0;
	if(choice3=="All images in a folder"){
		setBatchMode(true);
		imDir = getDirectory("Choose a Directory");
		dir = imDir+"Pixel_Matrix"+File.separator;
		File.makeDirectory(dir);
		files = getFileList(imDir);
		fileTypes = newArray("jpg","jpeg","tiff","bmp","png","gif");
		imNum=0;
		go=1;
		for(i=0;i<lengthOf(files);i++){
			if(File.isDirectory(imDir+files[i])==1){
				go=0;	
			}
			if(File.isDirectory(imDir+files[i])==0){
				go=0;
				for(j=0;j<lengthOf(fileTypes);j++){
					if(endsWith(imDir+files[i],fileTypes[j])){
						go=go+1;
					}
				}
				if(go>1)
					go=1;
			}
			if(go==1){
				if(imNum!=0){
					// Get the dimensions of the image to check if it's the same
					open(imDir+files[i]);
					getDimensions(width, height, channels, slices, frames);
					if(masterWidth-width!=0 || masterHeight-height!=0){
						close();
						print("Warning: <"+files[i]+"> does not have the same dimensions\nas the first image and has been skipped.");
						go=0;
					}else{
						imNum++;
						go=1;
					}
				}
				if(imNum==0){
					open(imDir+files[i]);
					// Get the dimensions of the image and create results array
					getDimensions(width, height, channels, slices, frames);
					masterWidth=width;			
					masterHeight=height;
					if(outputIn==1){
						resultsArray = newArray((width+1)*(height+1));
						Array.fill(resultsArray, 0);
					}
					if(outputCu==1){
						cumResultsArray  = newArray((width+1)*(height+1));
						Array.fill(cumResultsArray, 0);
					}
					if(outputFr==1||outputHM==1){
						freqResultsArray  = newArray((width+1)*(height+1));
						Array.fill(freqResultsArray, 0);
					}
					imNum=1;
					go=1;
				}
			}
			if(go==1){
				// Perform analysis
				run("8-bit");
				for(j=0; j<height; j++){
					for(k=0; k<width; k++){
						if(outputIn==1)
							resultsArray[(j*width)+k] = getPixel(k,j)/255;
						if(outputCu==1)
							cumResultsArray[(j*width)+k] = cumResultsArray[(j*width)+k] + getPixel(k,j)/255;
						if(outputFr==1||outputHM==1)
							freqResultsArray[(j*width)+k] = freqResultsArray[(j*width)+k] + getPixel(k,j)/255;
					}
				}
				if(outputIn==1){
					fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."));
					// Update results table
					for(row=0; row<height; row++){
						for(col=0; col<width; col++){
							setResult(col+1,row, resultsArray[(row*width)+col]);
						}
					}
					setOption("ShowRowNumbers", true);
					updateResults();
					// Note: row and column headers are also saved in the xls file
					saveAs("results", dir+fileName+".xls");	
				}
				close();
				print("File "+i+1+" completed...");
			}
		}
		print(imNum+" images processed successfully.");
		if(imNum==0){
			print("Error: no valid images were detected. This may be because the\nwrong folder was selected or your file types are not recognised.\nCurrently, only the below file types are supported:");
			toPrint = "";
			for(i=0;i<lengthOf(fileTypes);i++)
				toPrint = toPrint+"<"+fileTypes[i]+"> ";
			print(toPrint);
			print("Please ensure your file types are supported and that the correct\nfolder is selected when you restart the macro.");
			print("-----Macro Terminated-----");
			selectWindow("Log");
			exit("Error: no valid images were detected.\nSee Log for details.");
		}
		if(imNum==1){
			if(outputCu==1||outputFr==1||outputHM==1){
				invalids="";
				if(outputCu==1)
					invalids=invalids+"<Cumulative Matrix> ";
				if(outputFr==1)
					invalids=invalids+"<Frequency Matrix> ";
				if(outputHM==1)
					invalids=invalids+"<Heat Map>";
				outputCu = 0;
				outputFr = 0;
				outputHM = 0;
				print("Warning: only one valid image was detected. The following analyses\nhave been deselected:");
				print(invalids);
			}
		}
		setBatchMode(false);
		print("Creating results files...");
		if(outputCu==1){
			// Update results table for cumulative images
			run("Clear Results");
			for(row=0; row<masterHeight; row++){
				for(col=0; col<masterWidth; col++){
					setResult(col+1,row, cumResultsArray[(row*masterWidth)+col]);
				}
			}
			setOption("ShowRowNumbers", true);
			updateResults();
			// Save cumulative
			fileName = "all_images_cumulative";
			// Note: row and column headers are also saved in the xls file
			saveAs("results", dir+fileName+".xls");
		}
		if(outputFr==1){
			// Update results table for frequency ROI
			run("Clear Results");
//			for(i=0;i<lengthOf(freqResultsArray);i++){
//				freqResultsArrray[i] = freqResultsArrray[i]/imNum;
//			}
			for(row=0; row<masterHeight; row++){
				for(col=0; col<masterWidth; col++){
					setResult(col+1,row, freqResultsArray[(row*masterWidth)+col]/imNum);
				}
			}
			setOption("ShowRowNumbers", true);
			updateResults();
			// Save frequency
			fileName = "all_images_frequency";
			// Note: row and column headers are also saved in the xls file
			saveAs("results", dir+fileName+".xls");
		}
		if(outputHM==1){

			//Define colours, alternatively, the colour can be defined by: R*65536+G*256+B
			reds = newArray(11);
			Array.fill(reds,255);
			greens = newArray(255,255,255,255,227,200,173,146,97,48,0);
			blues = newArray(255,170,85,0,0,0,0,0,0,0,0);
			colours = newArray(11);
			for(i=0;i<11;i++){
				colours[i] = ((reds[i]&0xff) <<16)+((greens[i]&0xff)<<8)+(blues[i]&0xff);
			}

			//Set up windows for the new heat map and legend
			xLoc = (floor(screenWidth()/2))-(floor(masterWidth/2));
			yLoc = (floor(screenHeight()/2))-(floor(masterHeight/2));
			legFileName = "all_images_HeatMap_Legend.jpg";
			fileName = "all_images_HeatMap.jpg";

			drawHMLegend(xLoc, yLoc, masterHeight, colours, dir, legFileName);

			call("ij.gui.ImageWindow.setNextLocation",xLoc,yLoc);
			newImage(fileName, "RGB", masterWidth, masterHeight, 1);
			selectWindow(fileName);

			for(row=0; row<masterHeight; row++){
				for(col=0; col<masterWidth; col++){
					for(i=0;i<11;i++){
						if(round(freqResultsArray[(row*masterWidth)+col]/imNum*10)==i){
							setPixel(col,row,colours[i]);
						}
					}
				}
			}
			saveAs(".jpg",dir+fileName);
		}
	}//end of "all images in folder"
	if(choice3=="Manually open images"){
		more=1;
		imNum=0;
		go=1;
		while(more==1){
			call("ij.gui.ImageWindow.setNextLocation", 100, 100);
			waitForUser("In ImageJ, open the image to analyse,\nclose all other images and click 'OK'");
			if(imNum!=0){
				// Get the dimensions of the image to check if it's the same
				getDimensions(width, height, channels, slices, frames);
				if(masterWidth-width!=0 || masterHeight-height!=0){
					waitForUser("Error: this image does not have the\nsame dimensions as your first image.\nClick 'OK' to select another image.");
					go=0;
					more=1;
				}else{
					imNum++;
					go=1;
				}
			}
			if(imNum==0){
				getDimensions(width, height, channels, slices, frames);
				masterWidth=width;			
				masterHeight=height;
				// Get the dimensions of the image and create results array
				getDimensions(width, height, channels, slices, frames);
				if(outputIn==1){
					resultsArray = newArray((width+1)*(height+1));
					Array.fill(resultsArray, 0);
				}
				if(outputCu==1){
					cumResultsArray  = newArray((width+1)*(height+1));
					Array.fill(cumResultsArray, 0);
				}
				if(outputFr==1||outputHM==1){
					freqResultsArray  = newArray((width+1)*(height+1));
					Array.fill(freqResultsArray, 0);
				}
				imNum=1;
				go=1;
				dir = getDirectory("image")+"Pixel_Matrix"+File.separator;
				File.makeDirectory(dir);
			}			
			if(go==1){		
				// Perform analysis
				run("8-bit");
				for(j=0; j<height; j++){
					for(k=0; k<width; k++){
						if(outputIn==1)
							resultsArray[(j*width)+k] = getPixel(k,j)/255;
						if(outputCu==1)
							cumResultsArray[(j*width)+k] = cumResultsArray[(j*width)+k] + getPixel(k,j)/255;
						if(outputFr==1||outputHM==1)
							freqResultsArray[(j*width)+k] = freqResultsArray[(j*width)+k] + getPixel(k,j)/255;
					}
				}
				if(outputIn==1){
					fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."));
					// Update results table
					for(row=0; row<height; row++){
						for(col=0; col<width; col++){
							setResult(col+1,row, resultsArray[(row*width)+col]);
						}
					}
					setOption("ShowRowNumbers", true);
					updateResults();
					// Note: row and column headers are also saved in the xls file
					saveAs("results", dir+fileName+".xls");	
				}
				more=getBoolean("Do you have more images to analyse?");
			}
		}
		if(outputCu==1){
			// Update results table for cumulative images
			run("Clear Results");
			for(row=0; row<masterHeight; row++){
				for(col=0; col<masterWidth; col++){
					setResult(col+1,row, cumResultsArray[(row*masterWidth)+col]);
				}
			}
			setOption("ShowRowNumbers", true);
			updateResults();
			// Save cumulative
			fileName = "selected_images_cumulative";
			// Note: row and column headers are also saved in the xls file
			saveAs("results", dir+fileName+".xls");
		}
		if(outputFr==1){
			// Update results table for frequency ROI
			run("Clear Results");
//			for(i=0;i<lengthOf(freqResultsArray);i++){
//				freqResultsArrray[i] = freqResultsArrray[i]/imNum;
//			}
			for(row=0; row<masterHeight; row++){
				for(col=0; col<masterWidth; col++){
					setResult(col+1,row, freqResultsArray[(row*masterWidth)+col]/imNum);
				}
			}
			setOption("ShowRowNumbers", true);
			updateResults();
			// Save frequency
			fileName = "selected_images_frequency";
			// Note: row and column headers are also saved in the xls file
			saveAs("results", dir+fileName+".xls");
		}
		if(outputHM==1){

			//Define colours, alternatively, the colour can be defined by: R*65536+G*256+B
			reds = newArray(11);
			Array.fill(reds,255);
			greens = newArray(255,255,255,255,227,200,173,146,97,48,0);
			blues = newArray(255,170,85,0,0,0,0,0,0,0,0);
			colours = newArray(11);
			for(i=0;i<11;i++){
				colours[i] = ((reds[i]&0xff) <<16)+((greens[i]&0xff)<<8)+(blues[i]&0xff);
			}

			//Set up windows for the new heat map and legend
			xLoc = (floor(screenWidth()/2))-(floor(masterWidth/2));
			yLoc = (floor(screenHeight()/2))-(floor(masterHeight/2));
			legFileName = "selected_images_HeatMap_Legend.jpg";
			fileName = "selected_images_HeatMap.jpg";

			drawHMLegend(xLoc, yLoc, masterHeight, colours, dir, legFileName);

			call("ij.gui.ImageWindow.setNextLocation",xLoc,yLoc);
			newImage(fileName, "RGB", masterWidth, masterHeight, 1);
			selectWindow(fileName);

			for(row=0; row<masterHeight; row++){
				for(col=0; col<masterWidth; col++){
					for(i=0;i<11;i++){
						if(round(freqResultsArray[(row*masterWidth)+col]/imNum*10)==i){
							setPixel(col,row,colours[i]);
						}
					}
				}
			}
			saveAs(".jpg",dir+fileName);
		}
	} //end of "manually open"
} //end of "whole image>multiple images"

if(choice=="ROI(s) in the image" && choice2=="Single Image"){
	print(nROIs+" detected.");
	// Check that there are ROIs before proceeding
	if(nROIs == 0){
		print("Error: please add ROIs and restart the macro.\n-----Macro terminated-----");
		selectWindow("Log");
		exit("Error: No ROIs were detected.");
	}

	// Get size of ROI(s) to check that all ROIs are the same size 
	roiXs = newArray(nROIs);
	roiYs = newArray(nROIs);
	roiWidths = newArray(nROIs);
	roiHeights = newArray(nROIs);
	for(i=0; i<nROIs; i++){
		roiManager("Select", i);
		Roi.getBounds(x, y, width, height);
		roiXs[i] = x;
		roiYs[i] = y;
		roiWidths[i] = width;
		roiHeights[i] = height;
	}
	for(i=1; i<nROIs; i++){
		if(roiWidths[i] - roiWidths[0] != 0){
			print("Error: please make sure your ROIs are the same size and restart the macro.\n-----Macro terminated-----");
			selectWindow("Log");
			exit("Error: Your ROIs are not all the same size");
		}
		if(roiHeights[i] - roiHeights[0] != 0){
			print("Error: please make sure your ROIs are the same size and restart the macro.\n-----Macro terminated-----");
			selectWindow("Log");
			exit("Error: Your ROIs are not all the same size");	
		}
	}
	print("ROI(s) size detected: "+roiWidths[0]+" pixels (width) x "+roiHeights[0]+" pixels (height).");
	print("Requirements met.\nPerforming analysis ...");

	// Create results arrays and prepare directory to save files in
	if(outputIn==1){
		resultsArray = newArray((roiWidths[0]+1)*(roiHeights[0]+1));
		Array.fill(resultsArray, 0);
	}
	if(outputCu==1){
		cumResultsArray  = newArray((roiWidths[0]+1)*(roiHeights[0]+1));
		Array.fill(cumResultsArray, 0);
	}
	if(outputFr==1||outputHM==1){
		freqResultsArray  = newArray((roiWidths[0]+1)*(roiHeights[0]+1));
		Array.fill(freqResultsArray, 0);
	}
	dir = getDirectory("image")+"Pixel_Matrix"+File.separator;
	File.makeDirectory(dir);

	// Get the coordinates to analyse and perform analysis
	run("8-bit");
	for(i=0; i<nROIs; i++){
		roiManager("select",i);
		xPoints = newArray(roiWidths[i]);
		for(j=0; j<xPoints.length; j++){
			xPoints[j] = roiXs[i]+j;
		}
		yPoints = newArray(roiHeights[i]);
		for(j=0; j<yPoints.length; j++){
			yPoints[j] = roiYs[i]+j;
		}
		for(j=0; j<yPoints.length; j++){
			for(k=0; k<xPoints.length; k++){
				if(outputIn==1)
					resultsArray[(j*xPoints.length)+k] = getPixel(xPoints[k],yPoints[j])/255;
				if(outputCu==1)
					cumResultsArray[(j*xPoints.length)+k] = cumResultsArray[(j*xPoints.length)+k] + getPixel(xPoints[k],yPoints[j])/255;
				if(outputFr==1||outputHM==1)
					freqResultsArray[(j*xPoints.length)+k] = freqResultsArray[(j*xPoints.length)+k] + getPixel(xPoints[k],yPoints[j])/255/nROIs;
			}
		}
		if(outputIn==1){
			// Update results table for each ROI
			run("Clear Results");
			for(row=0; row<roiHeights[0]; row++){
				for(col=0; col<roiWidths[0]; col++){
					setResult(col+1,row, resultsArray[(row*roiWidths[0])+col]);
				}
			}
			setOption("ShowRowNumbers", true);
			updateResults();
			// Save each ROI
			fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."))+"_roi_"+getInfo("roi.name");
			// Note: row and column headers are also saved in the xls file
			saveAs("results", dir+fileName+".xls");
		}
	}
	if(outputCu==1){
		// Update results table for cumulative ROI
		run("Clear Results");
		for(row=0; row<roiHeights[0]; row++){
			for(col=0; col<roiWidths[0]; col++){
				setResult(col+1,row, cumResultsArray[(row*roiWidths[0])+col]);
			}
		}
		setOption("ShowRowNumbers", true);
		updateResults();
		// Save cumulative
		fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."))+"_allROIs_cumulative";
		// Note: row and column headers are also saved in the xls file
		saveAs("results", dir+fileName+".xls");
	}
	if(outputFr==1){
		// Update results table for frequency ROI
		run("Clear Results");
		for(row=0; row<roiHeights[0]; row++){
			for(col=0; col<roiWidths[0]; col++){
				setResult(col+1,row, freqResultsArray[(row*roiWidths[0])+col]);
			}
		}
		setOption("ShowRowNumbers", true);
		updateResults();
		// Save frequency
		fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."))+"_allROIs_frequency";
		// Note: row and column headers are also saved in the xls file
		saveAs("results", dir+fileName+".xls");
	}
	if(outputHM==1){

		//Define colours, alternatively, the colour can be defined by: R*65536+G*256+B
		reds = newArray(11);
		Array.fill(reds,255);
		greens = newArray(255,255,255,255,227,200,173,146,97,48,0);
		blues = newArray(255,170,85,0,0,0,0,0,0,0,0);
		colours = newArray(11);
		for(i=0;i<11;i++){
			colours[i] = ((reds[i]&0xff) <<16)+((greens[i]&0xff)<<8)+(blues[i]&0xff);
		}

		//Set up windows for the new heat map and legend
		xLoc = (floor(screenWidth()/2))-(floor(roiWidths[0]/2));
		yLoc = (floor(screenHeight()/2))-(floor(roiHeights[0]/2));
		legFileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."))+"_HeatMap_Legend.jpg";
		fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."))+"_HeatMap.jpg";

		drawHMLegend(xLoc, yLoc, roiHeights[0], colours, dir, legFileName);

		call("ij.gui.ImageWindow.setNextLocation",xLoc,yLoc);
		newImage(fileName, "RGB", roiWidths[0], roiHeights[0], 1);
		selectWindow(fileName);

		for(row=0; row<roiHeights[0]; row++){
			for(col=0; col<roiWidths[0]; col++){
				for(i=0;i<11;i++){
					if(round(freqResultsArray[(row*roiWidths[0])+col]*10)==i){
						setPixel(col,row,colours[i]);
					}
				}
			}
		}
		saveAs(".jpg",dir+fileName);
	}
}

if(choice=="ROI(s) in the image" && choice2=="Multiple Images"){
	masterHeight=0;
	masterWidth=0;
	more=1;
	roiNum=0;
	imNum=0;
	go=1;
	while(more==1){
//		call("ij.gui.ImageWindow.setNextLocation", 100, 100);
		waitForUser("In ImageJ, open the image to analyse,\nclose all other images, load the ROIs\ninto the ROI Manager and click 'OK'");
		nROIs = roiManager("count");
		while(nROIs==0){
			waitForUser("No ROIs were detected. Load ROIs\nto the ROI Manager and click 'OK'");
			nROIs = roiManager("count");
		}
// XXXXXXXXXXXXXXXX seems to be outputting data by image and not ROI
		imNum=imNum+1;
		if(imNum==1){
			dir = getDirectory("image")+"Pixel_Matrix"+File.separator;
			File.makeDirectory(dir);
		}
		for(i=0;i<nROIs;i++){
			roiManager("select",i);
			if(roiNum!=0){
				// Get the dimensions of the ROI to check if it's the same
				Roi.getBounds(x, y, width, height);	
				if(masterWidth-width!=0 || masterHeight-height!=0){
					fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."));
					roiName = getInfo("roi.name");				
					print("Warning: the following ROI\n<"+roiName+">\nin the following image\n<"+fileName+">\ndoes not have the same dimensions as your first image\nand has not been analysed.");
					go=0;
				}else{
					roiNum=roiNum+1;
					go=1;
				}
			}
			if(roiNum==0){
				Roi.getBounds(x, y, width, height);			
				masterWidth=width;			
				masterHeight=height;
				// Get the dimensions of the image and create results array
				if(outputIn==1){
					resultsArray = newArray((width+1)*(height+1));
					Array.fill(resultsArray, 0);
				}
				if(outputCu==1){
					cumResultsArray  = newArray((width+1)*(height+1));
					Array.fill(cumResultsArray, 0);
				}
				if(outputFr==1||outputHM==1){
					freqResultsArray  = newArray((width+1)*(height+1));
					Array.fill(freqResultsArray, 0);
				}
				xPoints = newArray(masterWidth);
				yPoints = newArray(masterHeight);
				roiNum=1;
				go=1;
			}
			if(go==1){		
				// Perform analysis
				run("8-bit");
				for(j=0; j<xPoints.length; j++){
					xPoints[j] = x+j;
				}
				for(j=0; j<yPoints.length; j++){
					yPoints[j] = y+j;
				}
				for(j=0; j<yPoints.length; j++){
					for(k=0; k<xPoints.length; k++){
						if(outputIn==1)
							resultsArray[(j*xPoints.length)+k] = getPixel(xPoints[k],yPoints[j])/255;
						if(outputCu==1)
							cumResultsArray[(j*xPoints.length)+k] = cumResultsArray[(j*xPoints.length)+k] + getPixel(xPoints[k],yPoints[j])/255;
						if(outputFr==1||outputHM==1)
							freqResultsArray[(j*xPoints.length)+k] = freqResultsArray[(j*xPoints.length)+k] + getPixel(xPoints[k],yPoints[j])/255;
					}
				}
				if(outputIn==1){
					fileName = substring(getTitle(), 0, lastIndexOf(getTitle(),"."));
					roiName = getInfo("roi.name");
					// Update results table
					for(row=0; row<height; row++){
						for(col=0; col<width; col++){
							setResult(col+1,row, resultsArray[(row*width)+col]);
						}
					}
					setOption("ShowRowNumbers", true);
					updateResults();
					// Note: row and column headers are also saved in the xls file
					saveAs("results", dir+fileName+"_"+roiName+".xls");	
				}
			}
		} // end of loop for each ROI
		more=getBoolean("Do you have more images to analyse?");
	}
	print(imNum+" images and "+roiNum+" ROIs analysed.");
	if(outputCu==1){
		// Update results table for cumulative images
		run("Clear Results");
		for(row=0; row<masterHeight; row++){
			for(col=0; col<masterWidth; col++){
				setResult(col+1,row, cumResultsArray[(row*masterWidth)+col]);
			}
		}
		setOption("ShowRowNumbers", true);
		updateResults();
		// Save cumulative
		fileName = "allRois_selectedImages_cumulative";
		// Note: row and column headers are also saved in the xls file
		saveAs("results", dir+fileName+".xls");
	}
	if(outputFr==1){
		// Update results table for frequency ROI
		run("Clear Results");
		for(row=0; row<masterHeight; row++){
			for(col=0; col<masterWidth; col++){
				setResult(col+1,row, freqResultsArray[(row*masterWidth)+col]/roiNum);
			}
		}
		setOption("ShowRowNumbers", true);
		updateResults();
		// Save frequency
		fileName = "allRois_selectedImages_frequency";
		// Note: row and column headers are also saved in the xls file
		saveAs("results", dir+fileName+".xls");
	}
	if(outputHM==1){
		//Define colours, alternatively, the colour can be defined by: R*65536+G*256+B
		reds = newArray(11);
		Array.fill(reds,255);
		greens = newArray(255,255,255,255,227,200,173,146,97,48,0);
		blues = newArray(255,170,85,0,0,0,0,0,0,0,0);
		colours = newArray(11);
		for(i=0;i<11;i++){
			colours[i] = ((reds[i]&0xff) <<16)+((greens[i]&0xff)<<8)+(blues[i]&0xff);
		}

		//Set up windows for the new heat map and legend
		xLoc = (floor(screenWidth()/2))-(floor(masterWidth/2));
		yLoc = (floor(screenHeight()/2))-(floor(masterHeight/2));
		legFileName = "allROIs_selectedImages_HeatMap_Legend.jpg";
		fileName = "allROIs_selectedImages_HeatMap.jpg";

		drawHMLegend(xLoc, yLoc, masterHeight, colours, dir, legFileName);
		call("ij.gui.ImageWindow.setNextLocation",xLoc,yLoc);
		newImage(fileName, "RGB", masterWidth, masterHeight, 1);
		selectWindow(fileName);

		for(row=0; row<masterHeight; row++){
			for(col=0; col<masterWidth; col++){
				for(i=0;i<11;i++){
					if(round(freqResultsArray[(row*masterWidth)+col]/roiNum*10)==i){
						setPixel(col,row,colours[i]);
					}
				}
			}
		}
		saveAs(".jpg",dir+fileName);
	}
}

if(isOpen("Results")==1){
	selectWindow("Results");
	run("Close");
}
print("Analysis complete."); 
if(outputIn==1||outputCu==1||outputFr==1)
	print("Excel file(s) saved to <"+dir+">.");
if(outputHM==1)
	print("Heat Map saved to <"+dir+">.");
print("-----End of Macro-----");
selectWindow("Log");
exit();

function drawHMLegend(xLoc, yLoc, roiHeight, colours, dir, legFileName){
		call("ij.gui.ImageWindow.setNextLocation",xLoc,yLoc+roiHeight+50);
		newImage(legFileName, "RGB white", 300, 40, 1);
		for(i=40;i<60;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[0]);
			}
		}
		for(i=60;i<80;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[1]);
			}
		}
		for(i=80;i<100;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[2]);
			}
		}
		for(i=100;i<120;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[3]);
			}
		}
		for(i=120;i<140;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[4]);
			}
		}
		for(i=140;i<160;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[5]);
			}
		}
		for(i=160;i<180;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[6]);
			}
		}
		for(i=180;i<200;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[7]);
			}
		}
		for(i=200;i<220;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[8]);
			}
		}
		for(i=220;i<240;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[9]);
			}
		}
		for(i=240;i<260;i++){
			for(j=10;j<20;j++){
				setPixel(i,j,colours[10]);
			}
		}
		setColor(0,0,0);
		setLineWidth(1);
		drawRect(40,10,220,10);
		drawString("0", 35, 35);
		drawString("0.5", 141, 35);
		drawString("1", 257, 35);
		saveAs(".jpg",dir+legFileName);
/*
//White					Yellow								Orange					Red
//0		1		2		3		4		5		6		7		8		9		10
//FFFFFF	FFFFAA	FFFF55	FFFF00	FFE300	FFC800	FFAD00	FF9200	FF6100	FF3000	FF0000	
//#ffffffff	#ffffffaa	#ffffff55	#ffffff00	#ffffe300	#ffffc800	#ffffad00	#ffff9200	#ffff6100	#ffff3000	#ffff0000
//R: 255	255		255		255		255		255		255		255		255		255		255
//G: 255	255		255		255		227		200		173		146		97		48		0
//B: 255	170		85		0		0		0		0		0		0		0		0
//Explanation: # is always there, next 2 chars is opacity, next 2 R, next 2 G, next 2 B; RGB values follow normal hex notation
//Colour gradient maker at: http://www.perbang.dk/rgbgradient/
//RGB to Hex converter at http://imagejdocu.tudor.lu/doku.php?id=macro:rgbtohex
//Code for visualising colours:
print("\\Clear");
newImage("trial.jpg", "RGB", 500, 500, 1);
reds = newArray(11);
Array.fill(reds,255);
greens = newArray(255,255,255,255,227,200,173,146,97,48,0);
blues = newArray(255,170,85,0,0,0,0,0,0,0,0);
colours = newArray(11);
for(i=0;i<11;i++){
	colours[i] = ((reds[i]&0xff) <<16)+((greens[i]&0xff)<<8)+(blues[i]&0xff);
	print("Colour for "+i/10+": "+colours[i]);
	for(j=0; j<500; j++){
		for(k=0;k<500;k++){
			setPixel(j,k,colours[i]);
		}
	}
	updateDisplay();
	print("-----");
	waitForUser;
}
*/
}
