/*
-------------------------------------------------------------------Title: Centroid Size Calculator Macro---------------------------------------------------------------
Version: 2.01
Updated: 17 Jan 2018

Description: 
- This macro calculates the averaged centroid size of all the ROIs in an image. WARNING: this can take a long time, especially if you have many ROIs that are defined 
by complex polygon ROIs.
- It outputs averaged centroid size (in pixels) and square pixels.

Requirements:
- A 2D ROI (i.e. rectangle, oval, polygon, traced or freehand) should be drawn around each ROI.
-----------------------------------------------------------------------------------------------------------------------------------------------------------------------
*/
print("\\Clear");
print("-----Centroid Size Calculator Macro Initiated-----");
print("Important Notes:\n -If you have many ROIs with many points, this could take awhile.");
print(" -All ROIs must be area ROIs (i.e. rectangle, oval, polygon, traced or freehand)");
print("Checking ROIs ...");

// Check that there are ROIs
nROIs = roiManager("count");
nGoodROIs=0;
if(nROIs == 0){
	print("Error: no ROIs detected. Add ROIs to your image and restart the macro.\n-----Macro Terminated-----");
	exit("Error: No ROIs detected.");
}
// Check that the ROIs are 2D
// 2D: rectangle, oval, polygon, freehand, traced
// 1D: line, angle, point, polyline, freeline
indicator = newArray(nROIs);
Array.fill(indicator, false);
goodTypes = newArray("polygon", "traced", "rectangle", "oval", "freehand");
for(i=0; i<nROIs; i++){
	roiManager("select", i);
	for(j=0; j<lengthOf(goodTypes); j++){
		test = goodTypes[j];
		if(Roi.getType == test)
			indicator[i] = 1;
		}
	if(indicator[i] == 1)
		nGoodROIs=nGoodROIs+1;
	if(indicator[i] == 0)
		print("Error: "+getInfo("selection.name")+" is a "+Roi.getType+" ROI, and is unsuitable for analysis.");
}
master = 1;
for(i=0; i<lengthOf(indicator); i++){
	master = indicator[i]*master;
}
if(master == 1)
	print("...\nRequirements met. Beginning analysis ...");
if(master == 0){
	Dialog.create("WARNING");
	Dialog.setInsets(0,5,0);
	Dialog.addMessage("Unsuitable ROI type detected, see Log for details.");
	Dialog.setInsets(10,0,0);
	Dialog.addMessage("Click 'OK' to continue (only the suitable ROI(s) will\nbe measured) or 'Cancel' to exit.");
	Dialog.show;
//	print("Please remove these ROIs or redraw them as an area ROI and restart the macro.\n-----Macro terminated-----");
//	exit("Error: unsuitable ROI type detected. See the Log window for details.");
	print("..\nContinuing analysis of suitable ROIs ...");
}

/* Code for checking if the ROIs have the same number of points (not used)
// Check that the ROI polygons have the same number of points
if(nROIs>1){
	indicator = 1;
	roiManager("select", 0);
	getSelectionCoordinates(x, y);
	masterName = getInfo("selection.name");
	masterLength = lengthOf(x);
	for(i=1; i<nROIs; i++){
		roiManager("select", i);
		getSelectionCoordinates(x, y);
		length = lengthOf(x);
		if(length != masterLength){
			indicator = 0;
			print("Warning: "+masterName+" and "+getInfo("selection.name")+" do not have the same number of points.");		
		}
	}
	if(indicator == 0){
//		print("Please ensure that the above ROIs have the same number of points and restart the macro.\n-----Macro terminated-----");
//		exit("Error: different number of points detected. See the Log window for details.");
		Dialog.create("WARNING");
		Dialog.setInsets(5,0,0);
		Dialog.addMessage("ROIs with different number of points have been detected (see Log window\nfor details). The analysis can still be performed but this is not recommended \nas it may be inaccurate.")
		Dialog.setInsets(5,0,0);
		Dialog.addMessage("Click OK to continue with the analysis or CANCEL to adjust your ROIs first.");
		Dialog.show();
	}
}
*/

run("Clear Results");

//Create arrays to store results
aveCentSize = newArray(nROIs);
pixelSize = newArray(nROIs); 

// Get all distances between points and centroid, calculate averaged centroid size and print results
for(i=0; i<nROIs; i++){
	roiManager("select", i);
	// If not correct type just print N.A. and skip
	if(indicator[i]==0){
		setResult("Label", i, getInfo("selection.name"));
		setResult("Averaged Centroid Size (px)", i, "N.A. (unsuitable ROI)");
		setResult("Size in Sq Pixels", i, "N.A. (unsuitable ROI)");
		continue;
	}
	// If correct, get size in square pixels
	getStatistics(pixelArea);
	pixelSize[i] = pixelArea;
	// Calculate ave centroid size
	getSelectionCoordinates(xpoints, ypoints);
	numPoints = lengthOf(xpoints);
	List.setMeasurements;
	x = List.getValue("X");
	y = List.getValue("Y");	
//	sqD = newArray(numPoints);
	sqD = 0;
	sumSqD = 0;
//	for(j=0; j<numPoints; j++){
//		sqD[j] = (xpoints[j]-x)*(xpoints[j]-x)+(ypoints[j]-y)*(ypoints[j]-y);
//		sumSqD = sumSqD+sqD[j];
//	}
	for(j=0; j<numPoints; j++){
		sqD = (xpoints[j]-x)*(xpoints[j]-x)+(ypoints[j]-y)*(ypoints[j]-y);
		sumSqD = sumSqD+sqD;
	}
/*For troubleshooting
print("---\nROI: "+getInfo("selection.name"));
print("-Points: "+numPoints);
print("-Centroid: "+x+","+y);
print("-xPoints:");
Array.print(xpoints);
print("-yPoints:");
Array.print(ypoints);
print("-sqDs:");
Array.print(sqD);
print("-sumSqD: "+sumSqD);
*/
	aveCentSize[i] = sqrt(sumSqD/sqrt(numPoints));
	setResult("Averaged Centroid Size (px)", i, aveCentSize[i]);
	setResult("Size in Sq Pixels", i, pixelSize[i]);
	setResult("Label", i, getInfo("selection.name"));
}

//Print Mean and SD to Results window
goodROIs = newArray(nGoodROIs);
skip=0;
for(i=0; i<nROIs; i++){
	if(indicator[i]==0)
		skip++;
	if(indicator[i]==1)
		goodROIs[i-skip]=i;
}
meanArray = newArray(nGoodROIs);
meanArray2 = newArray(nGoodROIs);
for(i=0; i<nGoodROIs; i++){
	meanArray[i]=aveCentSize[goodROIs[i]];
	meanArray2[i]=pixelSize[goodROIs[i]];
}
//Array.getStatistics(aveCentSize, min, max, mean, sd);
Array.getStatistics(meanArray, min, max, mean, sd);
setResult("Averaged Centroid Size (px)", nROIs, mean);
setResult("Averaged Centroid Size (px)", nROIs+1, sd);
Array.getStatistics(meanArray2, min, max, mean, sd);
setResult("Size in Sq Pixels", nROIs, mean);
setResult("Size in Sq Pixels", nROIs+1, sd);
setResult("Label", nROIs, "Mean");
setResult("Label", nROIs+1, "SD");
updateResults();

print("...\nCentroid Size results printed to Results window.\n-----End of Macro-----");
selectWindow("Log");
selectWindow("Results");
exit;
