/*
----------------------- Coverage Calculator Macro -----------------------
Version: 1.0
Updated: 13 May 2017

This macro will ask you to open 2 or 3 sets of ROIs to the ROI Manager in turn:
1) The ROI for the entire animal.
2) The ROI(s) representing the markings.
3) The ROI(s) representing any holes in the markings (these will be subtracted
from the markings). If this is not needed, simply click "No" when asked.
-------------------------------------------------------------------------
*/
print("\\Clear");
print("Coverage Calculator macro initiated ...\nCollecting options ...");
run("Clear Results");

//Measure total area, with error check for ROIs in the ROI Manager
waitForUser("Open the ROI(s) for the entire animal/pattern, then click OK:");
numROIs = roiManager("count");
if(numROIs<1)
	exit("Error: no ROIs detected in the ROI Manager.");
totArea = 0;
for(i=0;i<numROIs;i++){
	roiManager("select", i);
	getStatistics(area);
	totArea = totArea + area;
}
setResult("Total Area",0,totArea);

//Measure marking area, with error check for ROIs in the ROI Manager
waitForUser("Replace existing ROIs with the ROI(s) for the markings, then click OK:");
numROIs = roiManager("count");
if(numROIs<1)
	exit("Error: no ROIs detected in the ROI Manager.");
markArea = 0;
for(i=0;i<numROIs;i++){
	roiManager("select", i);
	getStatistics(area);
	markArea = markArea + area; 
}
setResult("Marking Area",0,markArea);


//Measure empty area, with error check for ROIs in the ROI Manager
Dialog.create("Empty Areas");
Dialog.addChoice("Are there empty areas in your markings to be subtracted?:",newArray("Yes","No"),"Yes");
Dialog.show();
choice = Dialog.getChoice();

if(choice=="No"){
	holeArea = 0;
	setResult("'Hole' Area",0,"N.A.");
}else if(choice=="Yes"){
	waitForUser("Replace existing ROI(s) with the ROI(s) for the empty areas/holes in the markings, then click OK:");
	numROIs = roiManager("count");
	if(numROIs<1)
		exit("Error: no ROIs detected in the ROI Manager.");
	holeArea = 0;
	for(i=0;i<numROIs;i++){
		roiManager("select", i);
		getStatistics(area);
		holeArea = holeArea + area;
	}
	setResult("'Hole' Area",0,holeArea);
}

//Calculate and output marking coverage result
setResult("Marking Coverage (%)",0,(markArea-holeArea)/totArea*100);

print("Marking coverage printed to Results table. \n-----End of Macro-----");
exit();

