/*
---------------------------------------------------Title: ROI Scaler and Randomiser--------------------------------------------------------
Version: 1.1
Updated: 21 Nov 2016

Description:
- This macro automatically scales and randomly positions ROIs around an image (or, if desired, a portion of the image).
- It saves the scaled ROI as ScaledROI.roi in the same folder as the original ROI.

Requirements:
- You need to have your ROI saved as a .roi file.
- If scaling is needed, you need to know the scale of the original and final image(s).
-------------------------------------------------------------------------------------------------------------------------------------------
*/

items = newArray("Whole image", "A portion of the image");
Dialog.create("ROI options");
Dialog.addNumber("Number of ROIs needed:",10);
Dialog.addMessage("----------------------")
Dialog.addMessage("Scaling")
Dialog.addNumber("Scale on picture where ROI was created:",1);
Dialog.addNumber("Scale on picture to randomise ROI:",1);
Dialog.addMessage("Note: If no scaling is required, leave both numbers as '1'.")
Dialog.addMessage("----------------------")
Dialog.addMessage("Positioning")
Dialog.addChoice("Do you want to place the ROIs around the entire image \nor just a portion of it?", items);
Dialog.show();
numROIs = Dialog.getNumber();
crSc = Dialog.getNumber();
bgSc = Dialog.getNumber();
choice = Dialog.getChoice();

if(choice=="A portion of the image"){
	setTool("rectangle");
	run("Select None");
	waitForUser("Draw a box around where you want your ROIs to be:");
	getSelectionBounds(x,y,width,height);
	selMinX = x;
	selMinY = y;
	selMaxX = x+width;
	selMaxY = y+height;
}

path = File.openDialog("Choose the saved ROI to use: \nNote: the scaled ROI will be saved in the same folder as ScaledROI.roi.");

roiManager("Open", path);
lastROI = roiManager("count");
roiManager("select", lastROI-1);
scale = d2s(bgSc/crSc, 4);
run("Scale... ", "x=&scale y=&scale centered");
roiManager("add");
roiManager("select", lastROI);
Roi.getBounds(x, y, width, height);
dir = File.getParent(path);
filePath = dir+File.separator+"ScaledROI.roi";
saveAs("Selection", filePath); 
secLastROI = lastROI-1;
roiManager("select", newArray(secLastROI,lastROI));
roiManager("delete");

if(choice=="Whole image"){
	minX = 1;
	minY = 1;
	maxX = getWidth()-width-1;
	maxY = getHeight()-height-1;
}else{
	minX = selMinX+1;
	minY = selMinY+1;
	maxX = selMaxX-width-1;
	maxY = selMaxY-height-1;
}

xCoord = newArray(numROIs);
for(i=0; i<numROIs; i++)
	xCoord[i] = ((maxX-minX)*random)+minX;
yCoord = newArray(numROIs);
for(i=0; i<numROIs; i++)
	yCoord[i] = ((maxY-minY)*random)+minY;

//xTrans = newArray(numROIs);
//for(i=0; i<numROIs; i++)
//	xTrans[i] = xCoord[i] - x;
//yTrans = newArray(numROIs);
//for(i=0; i<numROIs; i++)
//	yTrans[i] = yCoord[i] - y;

for(i=0; i<numROIs; i++){
	roiManager("Open", filePath);
	lastROI = roiManager("count");
	roiManager("select", lastROI-1);
	index = i+1;
	name = "r"+index;
	roiManager("Rename", name);
//	roiManager("translate", xTrans[i], yTrans[i]);
	Roi.move(xCoord[i], yCoord[i]);
}

exit("Ta-dah!");
